# Copyright (C) 2020 by ÿnérant, eichhornchen, nicomarg, charlse
# SPDX-License-Identifier: GPL-3.0-or-later

import gettext as gt
import subprocess
from pathlib import Path
from typing import Any, List


class Translator:
    """
    This module uses gettext to translate strings.
    Translator.setlocale defines the language of the strings,
    then gettext() translates the message.
    """
    SUPPORTED_LOCALES: List[str] = ["en", "fr"]
    locale: str = "en"
    translators: dict = {}

    for language in SUPPORTED_LOCALES:
        translators[language] = gt.translation(
            "squirrelbattle",
            localedir=Path(__file__).parent / "locale",
            languages=[language],
        )

    @classmethod
    def setlocale(cls, lang: str) -> None:
        """
        Define the language used to translate the game.
        The language must be supported, otherwise nothing is done.
        """
        lang = lang[:2]
        if lang in cls.SUPPORTED_LOCALES:
            cls.locale = lang

    @classmethod
    def get_translator(cls) -> Any:
        return cls.translators.get(cls.locale)

    @classmethod
    def makemessages(cls) -> None:  # pragma: no cover
        for language in cls.SUPPORTED_LOCALES:
            args = ["find", "squirrelbattle", "-iname", "*.py"]
            find = subprocess.Popen(args, cwd=Path(__file__).parent.parent,
                                    stdout=subprocess.PIPE)
            args = ["xargs", "xgettext", "--from-code", "utf-8",
                    "--join-existing",
                    "--add-comments",
                    "--package-name=squirrelbattle",
                    "--package-version=3.14.1",
                    "--copyright-holder=ÿnérant, eichhornchen, "
                    "nicomarg, charlse",
                    "--msgid-bugs-address=squirrel-battle@crans.org",
                    "-o", Path(__file__).parent / "locale" / language
                    / "LC_MESSAGES" / "squirrelbattle.po"]
            print(f"Make {language} messages...")
            subprocess.Popen(args, stdin=find.stdout)

    @classmethod
    def compilemessages(cls) -> None:  # pragma: no cover
        for language in cls.SUPPORTED_LOCALES:
            args = ["msgfmt", "--check-format",
                    "-o", Path(__file__).parent / "locale" / language
                    / "LC_MESSAGES" / "squirrelbattle.mo",
                    Path(__file__).parent / "locale" / language
                    / "LC_MESSAGES" / "squirrelbattle.po"]
            print(f"Compiling {language} messages...")
            subprocess.Popen(args)


def gettext(message: str) -> str:
    """
    Translate a message.
    """
    return Translator.get_translator().gettext(message)
